'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _nodeSass = require('node-sass');

var sass = _interopRequireWildcard(_nodeSass);

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

/**
 * The output directory for the Sass plugin
 */
var sassDir = '.magnet/sass';

/**
 * Flag indicating whether or not sass files have been
 * served yet.
 */
var isServingSassFiles = false;

/**
 * Serves built sass files using magnet server instance
 * @param {!Magnet} magnet
 * @param {!string} outputDir
 */
function serveSassFiles(magnet, outputDir) {
  if (!isServingSassFiles) {
    magnet.getServer().getEngine().use('/css', _express2.default.static(outputDir));
  }
  isServingSassFiles = true;
}

/**
 * Gets the output file path for a file and a given
 * output directory.
 * @param {!string} file
 * @param {!string} outputDir
 * @return {!string}
 */
function getOutputFile(file, outputDir) {
  var fileName = _path2.default.basename(file).replace('.scss', '.css');
  return _path2.default.join(outputDir, fileName);
}

/**
 * Writes css to file
 * @param {!string} file
 * @param {!string} css
 * @return {!Promise}
 */
function writeFile(file, css) {
  return new Promise(function (resolve, reject) {
    _fs2.default.writeFile(file, css, function (err) {
      if (err) {
        return reject(err);
      }
      resolve();
    });
  });
}

/**
 * Renders a single sass file
 * @param {!string} file
 * @param {!string} outputDir
 * @return {!Promise.<string>} The resulting css
 */
function renderFile(file) {
  return new Promise(function (resolve, reject) {
    sass.render({ file: file }, function (err, result) {
      if (err) {
        return reject(err);
      }
      resolve(result.css);
    });
  });
}

/**
 * Builds sass files to output dir
 * @param {!Array.<string>} files
 * @param {!string} outputDir
 * @return {!Promise}
 */
async function buildSassFiles(files, outputDir) {
  var renderedFiles = await Promise.all(files.map(async function (file) {
    var css = await renderFile(file);
    return { file: file, css: css };
  }));

  if (!_fs2.default.existsSync(outputDir)) {
    _fs2.default.mkdirSync(outputDir);
  }

  return Promise.all(renderedFiles.map(function (_ref) {
    var file = _ref.file,
        css = _ref.css;

    var outputFile = getOutputFile(file, outputDir);
    return writeFile(outputFile, css);
  }));
}

exports.default = {
  build: async function build(magnet) {
    var config = magnet.getConfig().magnet.pluginsConfig.sass;

    var files = config.src || [];
    if (!Array.isArray(files)) {
      files = [files];
    }

    if (!files.length) {
      return;
    }

    var outputDir = _path2.default.join(magnet.getDirectory(), sassDir);

    try {
      await buildSassFiles(files, outputDir);
    } catch (err) {
      throw new Error('Somethng went wrong compiling your stylesheets:\n' + err.message);
    }
  },
  start: function start(magnet) {
    var outputDir = _path2.default.join(magnet.getDirectory(), sassDir);
    serveSassFiles(magnet, outputDir);
  },
  test: function test() {
    return false;
  }
};